
import gaphor.adapters.connectors
import gaphor.adapters.editors

import gaphor.adapters.actions.flowconnect
import gaphor.adapters.actions.partitionpage
import gaphor.adapters.classes.classconnect
import gaphor.adapters.classes.interfaceconnect
import gaphor.adapters.components.connectorconnect
import gaphor.adapters.interactions.messageconnect
import gaphor.adapters.profiles.extensionconnect
import gaphor.adapters.profiles.stereotypespage
import gaphor.adapters.profiles.metaclasseditor
import gaphor.adapters.usecases.usecaseconnect

import gaphor.adapters.states.vertexconnect
import gaphor.adapters.states.propertypages
