"""
Font definitions.
"""

# Default font
FONT = 'sans 10'

# Default font for abstract named (e.g. attributes)
FONT_ABSTRACT = 'sans italic 10'

# Font for names of elements (such as classes)
FONT_NAME = 'sans bold 10'

# Abstract classes use this font for their name
FONT_ABSTRACT_NAME = 'sans bold italic 10'

# Small text, e.g. the (from ...) line in classes
FONT_SMALL = 'sans 8'

