extensions = ['sphinx.ext.autodoc', 'sphinx.ext.doctest', 'sphinx.ext.todo']
project = 'Gaphor'
version = '0.15'
release = version
source_suffix = '.txt'
master_doc = 'contents'
#html_theme = 'traditional'
#unused_docs = 'constraints.txt index.txt tools.txt comparison.txt'
